@@ -1,18 +1,19 @@
+# frozen_string_literal: true
+
 module Bundler
   class CLI::Check
     attr_reader :options
+
     def initialize(options)
       @options = options
     end
 
     def run
-      if options[:path]
-        Bundler.settings[:path] = File.expand_path(options[:path])
-        Bundler.settings[:disable_shared_gems] = "1"
-      end
+      Bundler.settings.set_command_option_if_given :path, options[:path]
+
       begin
         definition = Bundler.definition
-        definition.validate_ruby!
+        definition.validate_runtime!
         not_installed = definition.missing_specs
       rescue GemNotFound, VersionConflict
         Bundler.ui.error "Bundler can't satisfy your Gemfile's dependencies."
@@ -25,11 +26,11 @@ module Bundler
         not_installed.each {|s| Bundler.ui.error " * #{s.name} (#{s.version})" }
         Bundler.ui.warn "Install missing gems with `bundle install`"
         exit 1
-      elsif !Bundler.default_lockfile.exist? && Bundler.settings[:frozen]
+      elsif !Bundler.default_lockfile.file? && Bundler.frozen?
         Bundler.ui.error "This bundle has been frozen, but there is no #{Bundler.default_lockfile.relative_path_from(SharedHelpers.pwd)} present"
         exit 1
       else
-        Bundler.load.lock(:preserve_bundled_with => true) unless options[:"dry-run"]
+        Bundler.load.lock(:preserve_unknown_sections => true) unless options[:"dry-run"]
         Bundler.ui.info "The Gemfile's dependencies are satisfied"
       end
     end
